import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;
/**
 * A colored dot that moves in a straight line.
 * If a key is pressed while dot is over an arrow icon, award points.
 * This class provides methods used by the colored dots subclasses.
 * 
 * @author Athiwat Chunlakhan - Tun
 * @author Wairung Teeranalinvit - Ton
 * @author Soratouch Pronmaneerattanatri - MuDang
 * @version 2010.06.01
 */
public class Dots extends Actor
{
	/** points to award for PERFECT hit */
	public static final int PERFECT = 100;
	/** points to award for COOL hit */
	public static final int COOL = 80;
	/** points to award for NORMAL hit */
	public static final int NORMAL = 50;
	/** points to award for BAD hit */
	public static final int BAD = 20;
	/** points to award for a near miss */
	public static final int MISS = -40;
	
	/** range for PERFECT hit */
	public static final int PERFECT_DISTANCE = 20;
	/** range for COOL hit */
	public static final int COOL_DISTANCE = 40;
	/** range for NORMAL hit */
	public static final int NORMAL_DISTANCE = 60;
	/** range for BAD hit */
	public static final int BAD_DISTANCE = 80;
	/** range for miss hit */
	public static final int MISS_DISTANCE = 100;
	
	private final static double WALKING_SPEED = 5.0;
	
	/** indicates whether this dot has been killed already */
    private boolean killed = false;

	/**
     * Act - do whatever the Dots wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
	public void act() {
		// subclasses will override this
	}



	public void moveLeft() {
		double angle = Math.toRadians(getRotation());
		int x = (int) Math.round(getX() + Math.cos(angle) * WALKING_SPEED);
		int y = (int) Math.round(getY() + Math.sin(angle) * WALKING_SPEED);

		setLocation(x, y);
		if (atWorldEdge()) kill();
	}
	
	public void moveRight() {
		double angle = Math.toRadians(getRotation());
		int x = (int) Math.round(getX() - Math.cos(angle) * WALKING_SPEED);
		int y = (int) Math.round(getY() - Math.sin(angle) * WALKING_SPEED);

		setLocation(x, y);
		if (atWorldEdge()) kill();
	}

	public void moveUp() {
		double angle = Math.toRadians(getRotation() + 90);
		int x = (int) Math.round(getX() - Math.cos(angle) * WALKING_SPEED);
		int y = (int) Math.round(getY() - Math.sin(angle) * WALKING_SPEED);

		setLocation(x, y);
		if (atWorldEdge()) kill();
	}

	public void moveDown() {
		double angle = Math.toRadians(getRotation() - 90);
		int x = (int) Math.round(getX() - Math.cos(angle) * WALKING_SPEED);
		int y = (int) Math.round(getY() - Math.sin(angle) * WALKING_SPEED);

		setLocation(x, y);
		if (atWorldEdge()) kill();
	}

	/**
	 * Kill a dot - remove it from the world.
	 */
	public void kill() {
		killed = true;
		getWorld().removeObject(this);
	}
	
	/**
	 * Test if this dot has been killed yet.
	 * @return true if killed already
	 */
	public boolean killed() {
		return killed;
	}

	/**
	 * Test if this dot is near the edge of the world.
	 * @return true if at world edge.
	 */
	public boolean atWorldEdge() {
		if (getX() < 5 || getX() > getWorld().getWidth() - 5) return true;
		if (getY() < 5 || getY() > getWorld().getHeight() - 5) return true;
		else
			return false;
	}
	
	/**
	 * Test if this dot is near the edge of the world, 
	 * usually slightly larger value than atWorldEdge().
	 * @return true if at world edge.
	 */
	public boolean atWorldEdge1() {
		if (getX() < 6 || getX() > getWorld().getWidth() - 6) return true;
		if (getY() < 6 || getY() > getWorld().getHeight() - 6) return true;
		else
			return false;
	}

	/**
	 * Evaluate whether this dot is hitting an object of type clazz. 
	 * Return a non-zero score if this dot is within a a certain
	 * distance of an object of the clazz type (e.g. an Arrow).
	 * For example, to test if the dot is intersecting a left arrow use:
	 * <br><tt>
	 * int score = getHit( Left.class)
	 * </tt>
	 * @param clazz is the type of object we want to compare our object's position to.
	 * @return a score based on hit accuracy, one of the values PERFECT, COOL, NORMAL, BAD, MISS, 
	 *     or 0 if not within distance of a clazz object.
	 */
	public int getHit( Class clazz ) {
		List list = getObjectsInRange(PERFECT_DISTANCE, clazz);
		if ( list.size() > 0 ) return PERFECT;
		list = getObjectsInRange(COOL_DISTANCE, clazz);
		if ( list.size() > 0 ) return COOL;
		list = getObjectsInRange(NORMAL_DISTANCE, clazz);
		if ( list.size() > 0 ) return NORMAL;
		list = getObjectsInRange(BAD_DISTANCE, clazz);
		if ( list.size() > 0 ) return BAD;
		list = getObjectsInRange(MISS_DISTANCE, clazz);
		if ( list.size() > 0 ) return MISS;
		// otherwise return 0 - none of the above
		return 0;
	}

}